@@ -1,10 +1,10 @@
 /*
     MIDI Sequencer C++ library
-    Copyright (C) 2006-2019, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
@@ -16,20 +16,23 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <drumstick/alsaclient.h>
-#include <drumstick/alsaqueue.h>
-#include <drumstick/alsaevent.h>
+#include "errorcheck.h"
+#include <QCoreApplication>
 #include <QFile>
+#include <QReadLocker>
 #include <QRegExp>
 #include <QThread>
-#include <QReadLocker>
 #include <QWriteLocker>
+#include <drumstick/alsaclient.h>
+#include <drumstick/alsaevent.h>
+#include <drumstick/alsaqueue.h>
+
 #if defined(RTKIT_SUPPORT)
 #include <QDBusConnection>
 #include <QDBusInterface>
-#include <sys/types.h>
-#include <sys/syscall.h>
 #include <sys/resource.h>
+#include <sys/syscall.h>
+#include <sys/types.h>
 #endif
 #include <pthread.h>
 
@@ -53,16 +56,16 @@
 /**
  * @class QObject
  * The QObject class is the base class of all Qt objects.
- * @see   http://qt-project.org/doc/qt-5/qobject.html
+ * @see https://doc.qt.io/qt-5/qobject.html
  */
 
 /**
  * @class QThread
  * The QThread class provides platform-independent threads.
- * @see   http://qt-project.org/doc/qt-5/qthread.html
+ * @see https://doc.qt.io/qt-5/qthread.html
  */
 
-namespace drumstick {
+namespace drumstick { namespace ALSA {
 
 /**
  * @addtogroup ALSAClient
@@ -138,7 +141,7 @@ namespace drumstick {
  * is emitted. In any case, the event pointer must be deleted by the receiver
  * method.
  *
- * @see http://qt-project.org/doc/qt-5/threads-reentrancy.html
+ * @see https://doc.qt.io/qt-5/threads-reentrancy.html
  *
  * @section EventOutput Output
  *
@@ -175,8 +178,7 @@ namespace drumstick {
  * MidiClient::setOutputBufferSize() as well as MidiClient::getInputBufferSize()
  * and MidiClient::setInputBufferSize().
  *
- * @see http://www.alsa-project.org/alsa-doc/alsa-lib/group___seq_client.html
- * @}
+ * @see https://www.alsa-project.org/alsa-doc/alsa-lib/group___seq_client.html
  */
 
 /**
@@ -191,8 +193,8 @@ public:
         m_Wait(timeout),
         m_Stopped(false),
         m_RealTime(true) {}
-    virtual ~SequencerInputThread() {}
-    virtual void run();
+    virtual ~SequencerInputThread() = default;
+    void run() override;
     bool stopped();
     void stop();
     void setRealtimePriority();
@@ -213,10 +215,10 @@ public:
         m_NeedRefreshClientList(true),
         m_OpenMode(SND_SEQ_OPEN_DUPLEX),
         m_DeviceName("default"),
-        m_SeqHandle(0),
-        m_Thread(0),
-        m_Queue(0),
-        m_handler(0)
+        m_SeqHandle(nullptr),
+        m_Thread(nullptr),
+        m_Queue(nullptr),
+        m_handler(nullptr)
     { }
 
     bool m_eventsEnabled;
@@ -268,18 +270,15 @@ MidiClient::~MidiClient()
 {
     stopSequencerInput();
     detachAllPorts();
-    if (d->m_Queue != 0)
-        delete d->m_Queue;
+    delete d->m_Queue;
     close();
     freeClients();
-    if (d->m_Thread != 0)
-        delete d->m_Thread;
-    delete d;
+    delete d->m_Thread;
 }
 
-
 /**
  * Returns the sequencer handler managed by ALSA
+ * @return the sequencer handler
  */
 snd_seq_t*
 MidiClient::getHandle()
@@ -289,14 +288,16 @@ MidiClient::getHandle()
 
 /**
  * Returns true if the sequencer is opened
+ * @return wheter the sequencer is opened
  */
 bool MidiClient::isOpened()
 {
-    return (d->m_SeqHandle != NULL);
+    return !d.isNull() && (d->m_SeqHandle != nullptr);
 }
 
 /**
  * Returns the name of the sequencer device
+ * @return the device name
  */
 QString MidiClient::getDeviceName()
 {
@@ -305,6 +306,7 @@ QString MidiClient::getDeviceName()
 
 /**
  * Returns the last open mode used in open()
+ * @return the last open mode
  */
 int MidiClient::getOpenMode()
 {
@@ -313,6 +315,7 @@ int MidiClient::getOpenMode()
 
 /**
  * Returns the last block mode used in open()
+ * @return the last block mode
  */
 bool MidiClient::getBlockMode()
 {
@@ -321,6 +324,7 @@ bool MidiClient::getBlockMode()
 
 /**
  * Returns true if the events mode of delivery has been enabled
+ * @return whether the events mode of delivery is enabled
  */
 bool MidiClient::getEventsEnabled() const
 {
@@ -329,6 +333,7 @@ bool MidiClient::getEventsEnabled() const
 
 /**
  * Sets a sequencer event handler enabling the callback delivery mode
+ * @param handler the sequencer event handler
  */
 void MidiClient::setHandler(SequencerEventHandler* handler)
 {
@@ -346,7 +351,7 @@ void MidiClient::setHandler(SequencerEventHandler* handler)
  */
 void MidiClient::setRealTimeInput(bool enable)
 {
-    if (d->m_Thread == 0) {
+    if (d->m_Thread == nullptr) {
         d->m_Thread = new SequencerInputThread(this, DEFAULT_INPUT_TIMEOUT);
         d->m_Thread->m_RealTime = enable;
     }
@@ -359,7 +364,7 @@ void MidiClient::setRealTimeInput(bool enable)
  */
 bool MidiClient::realTimeInputEnabled()
 {
-    if (d->m_Thread == 0)
+    if (d->m_Thread == nullptr)
         return true;
     return d->m_Thread->m_RealTime;
 }
@@ -389,9 +394,9 @@ MidiClient::open( const QString deviceName,
                   const int openMode,
                   const bool blockMode)
 {
-    CHECK_ERROR( snd_seq_open( &d->m_SeqHandle, deviceName.toLocal8Bit().data(),
+    DRUMSTICK_ALSA_CHECK_ERROR( snd_seq_open( &d->m_SeqHandle, deviceName.toLocal8Bit().data(),
                               openMode, blockMode ? 0 : SND_SEQ_NONBLOCK ) );
-    CHECK_WARNING( snd_seq_get_client_info( d->m_SeqHandle, d->m_Info.m_Info ) );
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_get_client_info( d->m_SeqHandle, d->m_Info.m_Info ) );
     d->m_DeviceName = deviceName;
     d->m_OpenMode = openMode;
     d->m_BlockMode = blockMode;
@@ -423,12 +428,12 @@ MidiClient::open( snd_config_t* conf,
                   const int openMode,
                   const bool blockMode )
 {
-    CHECK_ERROR( snd_seq_open_lconf( &d->m_SeqHandle,
+    DRUMSTICK_ALSA_CHECK_ERROR( snd_seq_open_lconf( &d->m_SeqHandle,
                                      deviceName.toLocal8Bit().data(),
                                      openMode,
                                      blockMode ? 0 : SND_SEQ_NONBLOCK,
                                      conf ));
-    CHECK_WARNING( snd_seq_get_client_info(d->m_SeqHandle, d->m_Info.m_Info));
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_get_client_info(d->m_SeqHandle, d->m_Info.m_Info));
     d->m_DeviceName = deviceName;
     d->m_OpenMode = openMode;
     d->m_BlockMode = blockMode;
@@ -444,10 +449,10 @@ MidiClient::open( snd_config_t* conf,
 void
 MidiClient::close()
 {
-    if (d->m_SeqHandle != NULL) {
+    if (d->m_SeqHandle != nullptr) {
         stopSequencerInput();
-        CHECK_WARNING(snd_seq_close(d->m_SeqHandle));
-        d->m_SeqHandle = NULL;
+        DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_close(d->m_SeqHandle));
+        d->m_SeqHandle = nullptr;
     }
 }
 
@@ -477,7 +482,7 @@ void
 MidiClient::setOutputBufferSize(size_t newSize)
 {
     if (getOutputBufferSize() != newSize) {
-        CHECK_WARNING(snd_seq_set_output_buffer_size(d->m_SeqHandle, newSize));
+        DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_output_buffer_size(d->m_SeqHandle, newSize));
     }
 }
 
@@ -507,7 +512,7 @@ void
 MidiClient::setInputBufferSize(size_t newSize)
 {
     if (getInputBufferSize() != newSize) {
-        CHECK_WARNING(snd_seq_set_input_buffer_size(d->m_SeqHandle, newSize));
+        DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_input_buffer_size(d->m_SeqHandle, newSize));
     }
 }
 
@@ -526,9 +531,9 @@ MidiClient::setBlockMode(bool newValue)
     if (d->m_BlockMode != newValue)
     {
         d->m_BlockMode = newValue;
-        if (d->m_SeqHandle != NULL)
+        if (d->m_SeqHandle != nullptr)
         {
-            CHECK_WARNING(snd_seq_nonblock(d->m_SeqHandle, d->m_BlockMode ? 0 : 1));
+            DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_nonblock(d->m_SeqHandle, d->m_BlockMode ? 0 : 1));
         }
     }
 }
@@ -544,7 +549,7 @@ MidiClient::setBlockMode(bool newValue)
 int
 MidiClient::getClientId()
 {
-    return CHECK_WARNING(snd_seq_client_id(d->m_SeqHandle));
+    return DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_client_id(d->m_SeqHandle));
 }
 
 /**
@@ -582,10 +587,10 @@ MidiClient::doEvents()
 {
     do {
         int err = 0;
-        snd_seq_event_t* evp = NULL;
-        SequencerEvent* event = NULL;
+        snd_seq_event_t* evp = nullptr;
+        SequencerEvent* event = nullptr;
         err = snd_seq_event_input(d->m_SeqHandle, &evp);
-        if ((err >= 0) && (evp != NULL)) {
+        if ((err >= 0) && (evp != nullptr)) {
             switch (evp->type) {
 
             case SND_SEQ_EVENT_NOTE:
@@ -669,7 +674,7 @@ MidiClient::doEvents()
                 break;
             }
             // first, process the callback (if any)
-            if (d->m_handler != NULL) {
+            if (d->m_handler != nullptr) {
                 d->m_handler->handleSequencerEvent(event->clone());
             } else {
                 // second, process the event listeners
@@ -696,7 +701,7 @@ MidiClient::doEvents()
 void
 MidiClient::startSequencerInput()
 {
-    if (d->m_Thread == 0) {
+    if (d->m_Thread == nullptr) {
         d->m_Thread = new SequencerInputThread(this, DEFAULT_INPUT_TIMEOUT);
     }
     d->m_Thread->start( d->m_Thread->m_RealTime ?
@@ -710,7 +715,7 @@ void
 MidiClient::stopSequencerInput()
 {
     int counter = 0;
-    if (d->m_Thread != 0) {
+    if (d->m_Thread != nullptr) {
         if (d->m_Thread->isRunning()) {
             d->m_Thread->stop();
             while (!d->m_Thread->wait(500) && (counter < 10)) {
@@ -793,7 +798,7 @@ MidiClient::setThisClientInfo(const ClientInfo& val)
 void
 MidiClient::applyClientInfo()
 {
-    if (d->m_SeqHandle != NULL) {
+    if (d->m_SeqHandle != nullptr) {
         snd_seq_set_client_info(d->m_SeqHandle, d->m_Info.m_Info);
     }
 }
@@ -869,8 +874,8 @@ MidiClient::createPort()
 void
 MidiClient::portAttach(MidiPort* port)
 {
-    if (d->m_SeqHandle != NULL) {
-        CHECK_ERROR(snd_seq_create_port(d->m_SeqHandle, port->m_Info.m_Info));
+    if (d->m_SeqHandle != nullptr) {
+        DRUMSTICK_ALSA_CHECK_ERROR(snd_seq_create_port(d->m_SeqHandle, port->m_Info.m_Info));
         d->m_Ports.push_back(port);
     }
 }
@@ -882,13 +887,13 @@ MidiClient::portAttach(MidiPort* port)
 void
 MidiClient::portDetach(MidiPort* port)
 {
-    if (d->m_SeqHandle != NULL) {
+    if (d->m_SeqHandle != nullptr) {
         if(port->getPortInfo()->getClient() == getClientId())
         {
             return;
         }
-        CHECK_ERROR(snd_seq_delete_port(d->m_SeqHandle, port->getPortInfo()->getPort()));
-        port->setMidiClient(NULL);
+        DRUMSTICK_ALSA_CHECK_ERROR(snd_seq_delete_port(d->m_SeqHandle, port->getPortInfo()->getPort()));
+        port->setMidiClient(nullptr);
 
         MidiPortList::iterator it;
         for(it = d->m_Ports.begin(); it != d->m_Ports.end(); ++it)
@@ -907,12 +912,13 @@ MidiClient::portDetach(MidiPort* port)
  */
 void MidiClient::detachAllPorts()
 {
-    if (d->m_SeqHandle != NULL) {
-        MidiPortList::iterator it;
-        for (it = d->m_Ports.begin(); it != d->m_Ports.end(); ++it) {
-            CHECK_ERROR(snd_seq_delete_port(d->m_SeqHandle, (*it)->getPortInfo()->getPort()));
-            (*it)->setMidiClient(NULL);
-            d->m_Ports.erase(it);
+    if (d->m_SeqHandle != nullptr) {
+        QMutableListIterator<MidiPort*> it(d->m_Ports);
+        while (it.hasNext()) {
+            MidiPort* p = it.next();
+            DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_delete_port(d->m_SeqHandle, p->getPortInfo()->getPort()));
+            p->setMidiClient(nullptr);
+            it.remove();
         }
     }
 }
@@ -987,18 +993,18 @@ MidiClient::setErrorBounce(bool newValue)
 void
 MidiClient::output(SequencerEvent* ev, bool async, int timeout)
 {
-    int npfds;
-    pollfd* pfds;
+    pollfd* pfds = nullptr;
     if (async) {
-        CHECK_WARNING(snd_seq_event_output(d->m_SeqHandle, ev->getHandle()));
+        DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_event_output(d->m_SeqHandle, ev->getHandle()));
     } else {
-        npfds = snd_seq_poll_descriptors_count(d->m_SeqHandle, POLLOUT);
-        pfds = (pollfd*) alloca(npfds * sizeof(pollfd));
+        int npfds = snd_seq_poll_descriptors_count(d->m_SeqHandle, POLLOUT);
+        pfds = (pollfd*) calloc(npfds, sizeof(pollfd));
         snd_seq_poll_descriptors(d->m_SeqHandle, pfds, npfds, POLLOUT);
         while (snd_seq_event_output(d->m_SeqHandle, ev->getHandle()) < 0)
         {
             poll(pfds, npfds, timeout);
         }
+        free(pfds);
     }
 }
 
@@ -1015,18 +1021,17 @@ MidiClient::output(SequencerEvent* ev, bool async, int timeout)
  */
 void MidiClient::outputDirect(SequencerEvent* ev, bool async, int timeout)
 {
-    int npfds;
-    pollfd* pfds;
     if (async) {
-        CHECK_WARNING(snd_seq_event_output_direct(d->m_SeqHandle, ev->getHandle()));
+        DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_event_output_direct(d->m_SeqHandle, ev->getHandle()));
     } else {
-        npfds = snd_seq_poll_descriptors_count(d->m_SeqHandle, POLLOUT);
-        pfds = (pollfd*) alloca(npfds * sizeof(pollfd));
+        int npfds = snd_seq_poll_descriptors_count(d->m_SeqHandle, POLLOUT);
+        pollfd* pfds = (pollfd*) calloc(npfds, sizeof(pollfd));
         snd_seq_poll_descriptors(d->m_SeqHandle, pfds, npfds, POLLOUT);
         while (snd_seq_event_output_direct(d->m_SeqHandle, ev->getHandle()) < 0)
         {
             poll(pfds, npfds, timeout);
         }
+        free(pfds);
     }
 }
 
@@ -1041,7 +1046,7 @@ void MidiClient::outputDirect(SequencerEvent* ev, bool async, int timeout)
 void
 MidiClient::outputBuffer(SequencerEvent* ev)
 {
-    CHECK_WARNING(snd_seq_event_output_buffer(d->m_SeqHandle, ev->getHandle()));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_event_output_buffer(d->m_SeqHandle, ev->getHandle()));
 }
 
 /**
@@ -1057,18 +1062,17 @@ MidiClient::outputBuffer(SequencerEvent* ev)
  */
 void MidiClient::drainOutput(bool async, int timeout)
 {
-    int npfds;
-    pollfd* pfds;
     if (async) {
-        CHECK_WARNING(snd_seq_drain_output(d->m_SeqHandle));
+        DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_drain_output(d->m_SeqHandle));
     } else {
-        npfds = snd_seq_poll_descriptors_count(d->m_SeqHandle, POLLOUT);
-        pfds = (pollfd*) alloca(npfds * sizeof(pollfd));
+        int npfds = snd_seq_poll_descriptors_count(d->m_SeqHandle, POLLOUT);
+        pollfd* pfds = (pollfd*) calloc(npfds, sizeof(pollfd));
         snd_seq_poll_descriptors(d->m_SeqHandle, pfds, npfds, POLLOUT);
         while (snd_seq_drain_output(d->m_SeqHandle) < 0)
         {
             poll(pfds, npfds, timeout);
         }
+        free(pfds);
     }
 }
 
@@ -1091,7 +1095,7 @@ MidiClient::synchronizeOutput()
 MidiQueue*
 MidiClient::getQueue()
 {
-    if (d->m_Queue == NULL) {
+    if (d->m_Queue == nullptr) {
         createQueue();
     }
     return d->m_Queue;
@@ -1104,7 +1108,7 @@ MidiClient::getQueue()
 MidiQueue*
 MidiClient::createQueue()
 {
-    if (d->m_Queue != NULL) {
+    if (d->m_Queue != nullptr) {
         delete d->m_Queue;
     }
     d->m_Queue = new MidiQueue(this, this);
@@ -1120,7 +1124,7 @@ MidiClient::createQueue()
 MidiQueue*
 MidiClient::createQueue(QString const& queueName )
 {
-    if (d->m_Queue != NULL) {
+    if (d->m_Queue != nullptr) {
         delete d->m_Queue;
     }
     d->m_Queue = new MidiQueue(this, queueName, this);
@@ -1137,7 +1141,7 @@ MidiClient::createQueue(QString const& queueName )
 MidiQueue*
 MidiClient::useQueue(int queue_id)
 {
-    if (d->m_Queue != NULL) {
+    if (d->m_Queue != nullptr) {
         delete d->m_Queue;
     }
     d->m_Queue = new MidiQueue(this, queue_id, this);
@@ -1154,7 +1158,7 @@ MidiClient::useQueue(int queue_id)
 MidiQueue*
 MidiClient::useQueue(const QString& name)
 {
-    if (d->m_Queue != NULL) {
+    if (d->m_Queue != nullptr) {
         delete d->m_Queue;
     }
     int queue_id = getQueueId(name);
@@ -1173,7 +1177,7 @@ MidiClient::useQueue(const QString& name)
 MidiQueue*
 MidiClient::useQueue(MidiQueue* queue)
 {
-    if (d->m_Queue != NULL) {
+    if (d->m_Queue != nullptr) {
         delete d->m_Queue;
     }
     queue->setParent(this);
@@ -1342,7 +1346,7 @@ void
 MidiClient::setPoolInfo(const PoolInfo& info)
 {
     d->m_poolInfo = info;
-    CHECK_WARNING(snd_seq_set_client_pool(d->m_SeqHandle, d->m_poolInfo.m_Info));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_client_pool(d->m_SeqHandle, d->m_poolInfo.m_Info));
 }
 
 /**
@@ -1352,7 +1356,7 @@ MidiClient::setPoolInfo(const PoolInfo& info)
 void
 MidiClient::resetPoolInput()
 {
-    CHECK_WARNING(snd_seq_reset_pool_input(d->m_SeqHandle));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_reset_pool_input(d->m_SeqHandle));
 }
 
 /**
@@ -1362,7 +1366,7 @@ MidiClient::resetPoolInput()
 void
 MidiClient::resetPoolOutput()
 {
-    CHECK_WARNING(snd_seq_reset_pool_output(d->m_SeqHandle));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_reset_pool_output(d->m_SeqHandle));
 }
 
 /**
@@ -1372,7 +1376,7 @@ MidiClient::resetPoolOutput()
 void
 MidiClient::setPoolInput(int size)
 {
-    CHECK_WARNING(snd_seq_set_client_pool_input(d->m_SeqHandle, size));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_client_pool_input(d->m_SeqHandle, size));
 }
 
 /**
@@ -1382,7 +1386,7 @@ MidiClient::setPoolInput(int size)
 void
 MidiClient::setPoolOutput(int size)
 {
-    CHECK_WARNING(snd_seq_set_client_pool_output(d->m_SeqHandle, size));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_client_pool_output(d->m_SeqHandle, size));
 }
 
 /**
@@ -1392,7 +1396,7 @@ MidiClient::setPoolOutput(int size)
 void
 MidiClient::setPoolOutputRoom(int size)
 {
-    CHECK_WARNING(snd_seq_set_client_pool_output_room(d->m_SeqHandle, size));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_client_pool_output_room(d->m_SeqHandle, size));
 }
 
 /**
@@ -1402,7 +1406,7 @@ MidiClient::setPoolOutputRoom(int size)
 void
 MidiClient::dropInput()
 {
-    CHECK_WARNING(snd_seq_drop_input(d->m_SeqHandle));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_drop_input(d->m_SeqHandle));
 }
 
 /**
@@ -1412,7 +1416,7 @@ MidiClient::dropInput()
 void
 MidiClient::dropInputBuffer()
 {
-    CHECK_WARNING(snd_seq_drop_input_buffer(d->m_SeqHandle));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_drop_input_buffer(d->m_SeqHandle));
 }
 
 /**
@@ -1425,7 +1429,7 @@ MidiClient::dropInputBuffer()
 void
 MidiClient::dropOutput()
 {
-    CHECK_WARNING(snd_seq_drop_output(d->m_SeqHandle));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_drop_output(d->m_SeqHandle));
 }
 
 /**
@@ -1438,7 +1442,7 @@ MidiClient::dropOutput()
 void
 MidiClient::dropOutputBuffer()
 {
-    CHECK_WARNING(snd_seq_drop_output_buffer(d->m_SeqHandle));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_drop_output_buffer(d->m_SeqHandle));
 }
 
 /**
@@ -1450,7 +1454,7 @@ MidiClient::dropOutputBuffer()
 void
 MidiClient::removeEvents(const RemoveEvents* spec)
 {
-    CHECK_WARNING(snd_seq_remove_events(d->m_SeqHandle, spec->m_Info));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_remove_events(d->m_SeqHandle, spec->m_Info));
 }
 
 /**
@@ -1461,10 +1465,10 @@ SequencerEvent*
 MidiClient::extractOutput()
 {
     snd_seq_event_t* ev;
-    if (CHECK_WARNING(snd_seq_extract_output(d->m_SeqHandle, &ev) == 0)) {
+    if (DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_extract_output(d->m_SeqHandle, &ev) == 0)) {
         return new SequencerEvent(ev);
     }
-    return NULL;
+    return nullptr;
 }
 
 /**
@@ -1550,7 +1554,7 @@ unsigned short
 MidiClient::pollDescriptorsRevents(struct pollfd *pfds, unsigned int nfds)
 {
     unsigned short revents;
-    CHECK_WARNING( snd_seq_poll_descriptors_revents( d->m_SeqHandle,
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_poll_descriptors_revents( d->m_SeqHandle,
                                                      pfds, nfds,
                                                      &revents ));
     return revents;
@@ -1573,7 +1577,7 @@ MidiClient::_getDeviceName()
 void
 MidiClient::_setClientName(const char *name)
 {
-    CHECK_WARNING(snd_seq_set_client_name(d->m_SeqHandle, name));
+    DRUMSTICK_ALSA_CHECK_WARNING(snd_seq_set_client_name(d->m_SeqHandle, name));
 }
 
 /**
@@ -1588,7 +1592,7 @@ MidiClient::createSimplePort( const char *name,
                               unsigned int caps,
                               unsigned int type )
 {
-    return CHECK_WARNING( snd_seq_create_simple_port( d->m_SeqHandle,
+    return DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_create_simple_port( d->m_SeqHandle,
                                                       name, caps, type ));
 }
 
@@ -1599,7 +1603,7 @@ MidiClient::createSimplePort( const char *name,
 void
 MidiClient::deleteSimplePort(int port)
 {
-    CHECK_WARNING( snd_seq_delete_simple_port( d->m_SeqHandle, port ));
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_delete_simple_port( d->m_SeqHandle, port ));
 }
 
 /**
@@ -1611,7 +1615,7 @@ MidiClient::deleteSimplePort(int port)
 void
 MidiClient::connectFrom(int myport, int client, int port)
 {
-    CHECK_WARNING( snd_seq_connect_from(d->m_SeqHandle, myport, client, port ));
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_connect_from(d->m_SeqHandle, myport, client, port ));
 }
 
 /**
@@ -1623,7 +1627,7 @@ MidiClient::connectFrom(int myport, int client, int port)
 void
 MidiClient::connectTo(int myport, int client, int port)
 {
-    CHECK_WARNING( snd_seq_connect_to(d->m_SeqHandle, myport, client, port ));
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_connect_to(d->m_SeqHandle, myport, client, port ));
 }
 
 /**
@@ -1635,7 +1639,7 @@ MidiClient::connectTo(int myport, int client, int port)
 void
 MidiClient::disconnectFrom(int myport, int client, int port)
 {
-    CHECK_WARNING( snd_seq_disconnect_from(d->m_SeqHandle, myport, client, port ));
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_disconnect_from(d->m_SeqHandle, myport, client, port ));
 }
 
 /**
@@ -1647,7 +1651,7 @@ MidiClient::disconnectFrom(int myport, int client, int port)
 void
 MidiClient::disconnectTo(int myport, int client, int port)
 {
-    CHECK_WARNING( snd_seq_disconnect_to(d->m_SeqHandle, myport, client, port ));
+    DRUMSTICK_ALSA_CHECK_WARNING( snd_seq_disconnect_to(d->m_SeqHandle, myport, client, port ));
 }
 
 /**
@@ -1716,7 +1720,7 @@ MidiClient::SequencerInputThread::stop()
 }
 
 #if defined(RTKIT_SUPPORT)
-static pid_t _gettid(void) {
+static pid_t _gettid() {
     return (pid_t) ::syscall(SYS_gettid);
 }
 #endif
@@ -1741,9 +1745,9 @@ MidiClient::SequencerInputThread::setRealtimePriority()
     if (rt != 0) {
 #if defined(RTKIT_SUPPORT)
         const QString rtkit_service =
-                QLatin1String("org.freedesktop.RealtimeKit1");
+                QStringLiteral("org.freedesktop.RealtimeKit1");
         const QString rtkit_path =
-                QLatin1String("/org/freedesktop/RealtimeKit1");
+                QStringLiteral("/org/freedesktop/RealtimeKit1");
         const QString rtkit_iface = rtkit_service;
         thread = _gettid();
         QDBusConnection bus = QDBusConnection::systemBus();
@@ -1790,18 +1794,16 @@ MidiClient::SequencerInputThread::setRealtimePriority()
 void
 MidiClient::SequencerInputThread::run()
 {
-    unsigned long npfd;
-    pollfd* pfd;
-    if ( priority() == TimeCriticalPriority )
+    if ( priority() == TimeCriticalPriority ) {
         setRealtimePriority();
-
-    if (m_MidiClient != NULL) {
-        npfd = snd_seq_poll_descriptors_count(m_MidiClient->getHandle(), POLLIN);
-        pfd = (pollfd *) alloca(npfd * sizeof(pollfd));
+    }
+    if (m_MidiClient != nullptr) {
+        int npfd = snd_seq_poll_descriptors_count(m_MidiClient->getHandle(), POLLIN);
+        pollfd* pfd = (pollfd *) calloc(npfd, sizeof(pollfd));
         try
         {
             snd_seq_poll_descriptors(m_MidiClient->getHandle(), pfd, npfd, POLLIN);
-            while (!stopped() && (m_MidiClient != NULL))
+            while (!stopped() && (m_MidiClient != nullptr))
             {
                 int rt = poll(pfd, npfd, m_Wait);
                 if (rt > 0) {
@@ -1813,6 +1815,7 @@ MidiClient::SequencerInputThread::run()
         {
             qWarning() << "exception in input thread";
         }
+        free(pfd);
     }
 }
 
@@ -1883,6 +1886,8 @@ ClientInfo::clone()
 ClientInfo&
 ClientInfo::operator=(const ClientInfo& other)
 {
+    if (this == &other)
+        return *this;
     snd_seq_client_info_copy(m_Info, other.m_Info);
     m_Ports = other.m_Ports;
     return *this;
@@ -2175,6 +2180,8 @@ SystemInfo::clone()
 SystemInfo&
 SystemInfo::operator=(const SystemInfo& other)
 {
+    if (this == &other)
+        return *this;
     snd_seq_system_info_copy(m_Info, other.m_Info);
     return *this;
 }
@@ -2303,8 +2310,11 @@ PoolInfo::clone()
  * @param other Another PoolInfo object reference to be copied
  * @return This object
  */
-PoolInfo& PoolInfo::operator=(const PoolInfo& other)
+PoolInfo&
+PoolInfo::operator=(const PoolInfo& other)
 {
+    if (this == &other)
+        return *this;
     snd_seq_client_pool_copy(m_Info, other.m_Info);
     return *this;
 }
@@ -2492,4 +2502,30 @@ getRuntimeALSADriverNumber()
     return result;
 }
 
-} /* namespace drumstick */
+/**
+ * ALSA library version at build time.
+ *
+ * This string corresponds to the compilation library, which may be
+ * different to the runtime library.
+ * @return ALSA runtime library formatted as a QString
+ * @see getRuntimeALSALibraryVersion
+ */
+QString getCompiledALSALibraryVersion()
+{
+    return QStringLiteral(SND_LIB_VERSION_STR);
+}
+
+/**
+ * @brief getDrumstickLibraryVersion provides the Drumstick version as an edited QString
+ * @return Drumstick library version
+ */
+QString getDrumstickLibraryVersion()
+{
+    return QStringLiteral(QT_STRINGIFY(VERSION));
+}
+
+/** @} */
+
+} // namespace ALSA
+} // namespace drumstick
+
