@@ -1,10 +1,10 @@
 /*
     Drumstick MIDI realtime input-output
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,16 +16,15 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <QDebug>
-#include <drumstick/rtmidioutput.h>
 #include "midiparser.h"
+#include <drumstick/rtmidioutput.h>
 
 namespace drumstick {
 namespace rt {
 
 class MIDIParser::MIDIParserPrivate {
 public:
-    MIDIParserPrivate(): m_in(0), m_out(0), m_running_status(0) { }
+    MIDIParserPrivate(): m_in(nullptr), m_out(nullptr), m_running_status(0) { }
     MIDIInput *m_in;
     MIDIOutput *m_out;
     unsigned char m_running_status;
@@ -34,111 +33,110 @@ public:
     void processNoteOff(const int chan, const int note, const int vel)
     {
         //qDebug() << "NoteOff(" << hex << chan << "," << note << "," << vel << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendNoteOff(chan, note, vel);
         }
-        if (m_in != 0) {
-            m_in->emit midiNoteOff(chan, note, vel);
+        if (m_in != nullptr) {
+            emit m_in->midiNoteOff(chan, note, vel);
         }
     }
 
     void processNoteOn(const int chan, const int note, const int vel)
     {
         //qDebug() << "NoteOn(" << hex << chan << "," << note << "," << vel << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendNoteOn(chan, note, vel);
         }
-        if (m_in != 0) {
-            m_in->emit midiNoteOn(chan, note, vel);
+        if (m_in != nullptr) {
+            emit m_in->midiNoteOn(chan, note, vel);
         }
     }
 
     void processKeyPressure(const int chan, const int note, const int value)
     {
         //qDebug() << "KeyPressure(" << hex << chan << "," << note << "," << value << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendKeyPressure(chan, note, value);
         }
-        if (m_in != 0) {
-            m_in->emit midiKeyPressure(chan, note, value);
+        if (m_in != nullptr) {
+            emit m_in->midiKeyPressure(chan, note, value);
         }
     }
 
     void processController(const int chan, const int control, const int value)
     {
         //qDebug() << "Controller(" << chan << "," << control << "," << value << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendController(chan, control, value);
         }
-        if (m_in != 0) {
-            m_in->emit midiController(chan, control, value);
+        if (m_in != nullptr) {
+            emit m_in->midiController(chan, control, value);
         }
     }
 
     void processProgram(const int chan, const int program)
     {
         //qDebug() << "Program(" << hex << chan << "," << program << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendProgram(chan, program);
         }
-        if (m_in != 0) {
-            m_in->emit midiProgram(chan, program);
+        if (m_in != nullptr) {
+            emit m_in->midiProgram(chan, program);
         }
     }
 
     void processChannelPressure(const int chan, const int value)
     {
         //qDebug() << "ChannelPressure(" << chan << "," << value << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendChannelPressure(chan, value);
         }
-        if (m_in != 0) {
-            m_in->emit midiChannelPressure(chan, value);
+        if (m_in != nullptr) {
+            emit m_in->midiChannelPressure(chan, value);
         }
     }
 
     void processPitchBend(const int chan, const int value)
     {
         //qDebug() << "PitchBend(" << chan << "," << value << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendPitchBend(chan, value);
         }
-        if (m_in != 0) {
-            m_in->emit midiPitchBend(chan, value);
+        if (m_in != nullptr) {
+            emit m_in->midiPitchBend(chan, value);
         }
     }
 
     void processSysex(const QByteArray &data)
     {
         //qDebug() << "Sysex(" << data.toHex() << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendSysex(data);
         }
-        if (m_in != 0) {
-            m_in->emit midiSysex(data);
+        if (m_in != nullptr) {
+            emit m_in->midiSysex(data);
         }
     }
 
     void processSystemCommon(const int status)
     {
         //qDebug() << "common SystemMsg(" << hex << status << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendSystemMsg(status);
         }
-        if (m_in != 0) {
-            m_in->emit midiSystemCommon(status);
-
+        if (m_in != nullptr) {
+            emit m_in->midiSystemCommon(status);
         }
     }
 
     void processSystemRealtime(unsigned char byte)
     {
         //qDebug() << "realtime SystemMsg(" << hex << byte << ")";
-        if (m_in != 0 && m_in->isEnabledMIDIThru() && m_out != 0) {
+        if (m_in != nullptr && m_in->isEnabledMIDIThru() && m_out != nullptr) {
             m_out->sendSystemMsg(byte);
         }
-        if (m_in != 0) {
-            m_in->emit midiSystemRealtime(byte);
+        if (m_in != nullptr) {
+            emit m_in->midiSystemRealtime(byte);
         }
     }
 
@@ -164,9 +162,6 @@ void MIDIParser::setMIDIThruDevice(MIDIOutput *device)
 
 void MIDIParser::parse(unsigned char byte)
 {
-    unsigned char status;
-    int chan, m1, m2, v;
-
     if (byte >= MIDI_STATUS_REALTIME) { // system realtime
         d->processSystemRealtime(byte);
         return;
@@ -174,7 +169,8 @@ void MIDIParser::parse(unsigned char byte)
         d->m_buffer.append(byte);
 
     while(d->m_buffer.length() > 0) {
-        status = static_cast<unsigned>(d->m_buffer.at(0));
+        int chan, m1, m2, v;
+        unsigned char status = static_cast<unsigned>(d->m_buffer.at(0));
         if (status == MIDI_STATUS_SYSEX) { // system exclusive
             if (byte == MIDI_STATUS_ENDSYSEX) {
                 d->processSysex(d->m_buffer);
@@ -256,5 +252,6 @@ void MIDIParser::parse(QByteArray bytes)
     }
 }
 
-}}
+} // namespace rt
+} // namespace drumstick
 
