@@ -1,10 +1,10 @@
 /*
     Drumstick RT (realtime MIDI In/Out)
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -28,12 +28,12 @@ void DummyInput::initialize(QSettings* settings)
 
 QString DummyInput::backendName()
 {
-    return "DUMMY";
+    return QStringLiteral("DUMMY");
 }
 
 QString DummyInput::publicName()
 {
-    return QString();
+    return QStringLiteral("DUMMY In");
 }
 
 void DummyInput::setPublicName(QString name)
@@ -41,10 +41,10 @@ void DummyInput::setPublicName(QString name)
     Q_UNUSED(name)
 }
 
-QStringList DummyInput::connections(bool advanced)
+QList<MIDIConnection> DummyInput::connections(bool advanced)
 {
     Q_UNUSED(advanced)
-    return QStringList();
+    return QList<MIDIConnection>();
 }
 
 void DummyInput::setExcludedConnections(QStringList conns)
@@ -52,14 +52,14 @@ void DummyInput::setExcludedConnections(QStringList conns)
     Q_UNUSED(conns)
 }
 
-QString DummyInput::currentConnection()
+MIDIConnection DummyInput::currentConnection()
 {
-    return QString();
+    return MIDIConnection();
 }
 
-void DummyInput::open(QString name)
+void DummyInput::open(const MIDIConnection& conn)
 {
-    Q_UNUSED(name)
+    Q_UNUSED(conn)
 }
 
 void DummyInput::close()
