@@ -1,10 +1,10 @@
 /*
     Drumstick RT (realtime MIDI In/Out)
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -29,13 +29,11 @@ namespace rt {
     class DummyInput : public MIDIInput
     {
         Q_OBJECT
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-        Q_PLUGIN_METADATA(IID "net.sourceforge.drumstick.rt.MIDIInput")
-#endif
+        Q_PLUGIN_METADATA(IID "net.sourceforge.drumstick.rt.MIDIInput/2.0")
         Q_INTERFACES(drumstick::rt::MIDIInput)
     public:
-        DummyInput(QObject *parent = 0) : MIDIInput(parent) {}
-        virtual ~DummyInput() {}
+        explicit DummyInput(QObject *parent = nullptr) : MIDIInput(parent) {}
+        virtual ~DummyInput() = default;
 
         // MIDIInput interface
     public:
@@ -43,11 +41,11 @@ namespace rt {
         virtual QString backendName();
         virtual QString publicName();
         virtual void setPublicName(QString name);
-        virtual QStringList connections(bool advanced);
+        virtual QList<MIDIConnection> connections(bool advanced);
         virtual void setExcludedConnections(QStringList conns);
-        virtual void open(QString name);
+        virtual void open(const MIDIConnection& name);
         virtual void close();
-        virtual QString currentConnection();
+        virtual MIDIConnection currentConnection();
 
         virtual void setMIDIThruDevice(MIDIOutput *device);
         virtual void enableMIDIThru(bool enable);
