@@ -1,10 +1,10 @@
 /*
     Drumstick RT (realtime MIDI In/Out)
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -28,12 +28,12 @@ void DummyOutput::initialize(QSettings* settings)
 
 QString DummyOutput::backendName()
 {
-    return QLatin1String("DUMMY");
+    return QStringLiteral("DUMMY");
 }
 
 QString DummyOutput::publicName()
 {
-    return QString();
+    return QStringLiteral("DUMMY Out");
 }
 
 void DummyOutput::setPublicName(QString name)
@@ -41,10 +41,10 @@ void DummyOutput::setPublicName(QString name)
     Q_UNUSED(name)
 }
 
-QStringList DummyOutput::connections(bool advanced)
+QList<MIDIConnection> DummyOutput::connections(bool advanced)
 {
     Q_UNUSED(advanced)
-    return QStringList();
+    return QList<MIDIConnection>();
 }
 
 void DummyOutput::setExcludedConnections(QStringList conns)
@@ -52,18 +52,18 @@ void DummyOutput::setExcludedConnections(QStringList conns)
     Q_UNUSED(conns)
 }
 
-void DummyOutput::open(QString name)
+void DummyOutput::open(const MIDIConnection& conn)
 {
-    Q_UNUSED(name)
+    Q_UNUSED(conn)
 }
 
 void DummyOutput::close()
 {
 }
 
-QString DummyOutput::currentConnection()
+MIDIConnection DummyOutput::currentConnection()
 {
-    return QString();
+    return MIDIConnection();
 }
 
 void DummyOutput::sendNoteOff(int chan, int note, int vel)
