@@ -1,10 +1,10 @@
 /*
     Drumstick RT Backend
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -20,10 +20,9 @@
 #include "macmidiinput.h"
 #include "maccommon.h"
 
-#include <QDebug>
 #include <QStringList>
 #include <QMutex>
-#include <QTextCodec>
+//#include <QTextCodec>
 #include <QObject>
 #include <QtConcurrent>
 
@@ -35,8 +34,8 @@ namespace rt {
 
     static CFStringRef DEFAULT_PUBLIC_NAME CFSTR("MIDI In");
 
-    void MacMIDIReadProc( const MIDIPacketList *pktlist,
-                          void *refCon, void *connRefCon );
+    static void MacMIDIReadProc( const MIDIPacketList *pktlist,
+                                 void *refCon, void *connRefCon );
 
     class MacMIDIInputPrivate {
     public:
@@ -49,11 +48,13 @@ namespace rt {
         bool m_thruEnabled;
         bool m_clientFilter;
         QString m_publicName;
-        QString m_currentInput;
+        MIDIConnection m_currentInput;
         QStringList m_excludedNames;
-        QStringList m_inputDevices;
+        QList<MIDIConnection> m_inputDevices;
+        bool m_status;
+        QStringList m_diagnostics;
 
-        MacMIDIInputPrivate(MacMIDIInput *inp) :
+        explicit MacMIDIInputPrivate(MacMIDIInput *inp) :
             m_inp(inp),
             m_out(nullptr),
             m_client(0),
@@ -67,22 +68,30 @@ namespace rt {
             internalCreate( DEFAULT_PUBLIC_NAME );
         }
 
+        void registerStatus(const QString& context, const OSStatus status)
+        {
+            if (status != noErr) {
+                m_diagnostics << QString("%1 error: %2").arg(context, status);
+                m_diagnostics << getErrorTextFromOSStatus(status);
+            }
+        }
+
         void internalCreate(CFStringRef name)
         {
             OSStatus result = noErr;
             result = MIDIClientCreate( name , nullptr, nullptr, &m_client );
             if (result != noErr) {
-                qDebug() << "MIDIClientCreate() err:" << result;
+                registerStatus("MIDIClientCreate()", result);
                 return;
             }
             result = MIDIDestinationCreate ( m_client, name, MacMIDIReadProc, (void*) this, &m_endpoint );
             if (result != noErr) {
-                qDebug() << "MIDIDestinationCreate() err:" << result;
+                registerStatus("MIDIDestinationCreate()", result);
                 return;
             }
             result = MIDIInputPortCreate( m_client, name, MacMIDIReadProc, (void *) this,  &m_port );
             if (result != noErr) {
-                qDebug() << "MIDIInputPortCreate() error:" << result;
+                registerStatus("MIDIInputPortCreate()", result);
                 return;
             }
             reloadDeviceList(true);
@@ -99,21 +108,21 @@ namespace rt {
             if (m_port != 0) {
                 result = MIDIPortDispose(m_port);
                 if (result != noErr) {
-                    qDebug() << "MIDIPortDispose() error:" << result;
+                   registerStatus("MIDIPortDispose()", result);
                     m_port = 0;
                 }
             }
             if (m_endpoint != 0) {
                 result = MIDIEndpointDispose(m_endpoint);
                 if (result != noErr) {
-                    qDebug() << "MIDIEndpointDispose() err:" << result;
+                    registerStatus("MIDIEndpointDispose()", result);
                     m_endpoint = 0;
                 }
             }
             if (m_client != 0) {
                 result = MIDIClientDispose(m_client);
                 if (result != noErr) {
-                    qDebug() << "MIDIClientDispose() err:" << result;
+                    registerStatus("MIDIClientDispose()", result);
                     m_client = 0;
                 }
             }
@@ -130,23 +139,24 @@ namespace rt {
                 if (dest != 0) {
                     QString name = getEndpointName(dest);
                     if ( m_clientFilter &&
-                         name.contains(QLatin1String("IAC"), Qt::CaseSensitive) )
+                         name.contains(QStringLiteral("IAC"), Qt::CaseSensitive) )
                         continue;
                     if ( name.contains(m_publicName))
                         continue;
-                    foreach( const QString& n, m_excludedNames) {
+                    for (const QString& n : m_excludedNames) {
                         if (name.contains(n)) {
                             excluded = true;
                             break;
                         }
                     }
-                    if (!excluded)
-                        m_inputDevices << name;
+                    if (!excluded) {
+                        m_inputDevices << MIDIConnection(name, i);
+                    }
                 }
             }
-            if (!m_currentInput.isEmpty() && m_source != 0 &&
+            if (!m_currentInput.first.isEmpty() && m_source != 0 &&
                 !m_inputDevices.contains(m_currentInput)) {
-                m_currentInput.clear();
+                m_currentInput = MIDIConnection();
                 m_source = 0;
             }
         }
@@ -160,39 +170,35 @@ namespace rt {
             }
         }
 
-        void open(QString name)
+        void open(const MIDIConnection& conn)
         {
-            int index = -1;
             OSStatus result = noErr;
-            QStringList allInputDevices;
-            int num = MIDIGetNumberOfSources();
-            for (int i = 0; i < num; ++i) {
-                MIDIEndpointRef dest = MIDIGetSource( i );
-                if (dest != 0)
-                   allInputDevices << getEndpointName( dest );
-            }
-            index = allInputDevices.indexOf(name);
-            if (index < 0)
-                return;
-            m_source = MIDIGetSource( index );
+            m_diagnostics.clear();
+            m_status = false;
+            m_source = MIDIGetSource( conn.second.toInt() );
             result = MIDIPortConnectSource( m_port, m_source, nullptr );
             if (result != noErr) {
-                qDebug() << "MIDIPortConnectSource() error:" << result;
+                registerStatus("MIDIPortConnectSource()", result);
                 return;
             }
-            m_currentInput = name;
+            m_currentInput = conn;
+            m_status = (result == noErr);
             return;
         }
 
         void close()
         {
             OSStatus result = noErr;
+            m_status = false;
+            m_diagnostics.clear();
             if (m_source != 0) {
                 result = MIDIPortDisconnectSource(m_port, m_source);
-                if (result != noErr)
-                    qDebug() << "MIDIPortDisconnectSource() error:" << result;
+                if (result != noErr) {
+                    registerStatus("MIDIPortDisconnectSource()", result);
+                }
                 m_source = 0;
-                m_currentInput.clear();
+                m_currentInput = MIDIConnection();
+                m_status = (result == noErr);
             }
         }
 
@@ -254,13 +260,13 @@ namespace rt {
                     j+=packet.length();
                     break;
                 default:
-                    qDebug() << "status?" << status;
+                    registerStatus("invalid status", status);
                 }
             }
         }
     };
 
-    void MacMIDIReadProc( const MIDIPacketList *pktlist, void *refCon, void *connRefCon )
+    static void MacMIDIReadProc( const MIDIPacketList *pktlist, void *refCon, void *connRefCon )
     {
         Q_UNUSED(connRefCon)
         MacMIDIInputPrivate  *obj = nullptr;
@@ -270,7 +276,11 @@ namespace rt {
         for (unsigned int i = 0; i < pktlist->numPackets; ++i) {
             if (obj != nullptr && packet != nullptr) {
                 QByteArray data((const char *)packet->data, packet->length);
+#if (QT_VERSION < QT_VERSION_CHECK(6,0,0))
                 QtConcurrent::run(obj, &MacMIDIInputPrivate::emitSignals, data);
+#else
+                obj->emitSignals(data);
+#endif
             }
             packet = MIDIPacketNext(packet);
         }
@@ -293,7 +303,7 @@ namespace rt {
 
     QString MacMIDIInput::backendName()
     {
-        return QLatin1Literal("CoreMIDI");
+        return QLatin1String("CoreMIDI");
     }
 
     QString MacMIDIInput::publicName()
@@ -306,7 +316,7 @@ namespace rt {
         d->setPublicName(name);
     }
 
-    QStringList MacMIDIInput::connections(bool advanced)
+    QList<MIDIConnection> MacMIDIInput::connections(bool advanced)
     {
         d->reloadDeviceList(advanced);
         return d->m_inputDevices;
@@ -317,9 +327,9 @@ namespace rt {
         d->m_excludedNames = conns;
     }
 
-    void MacMIDIInput::open(QString name)
+    void MacMIDIInput::open(const MIDIConnection& conn)
     {
-        d->open(name);
+        d->open(conn);
     }
 
     void MacMIDIInput::close()
@@ -327,7 +337,7 @@ namespace rt {
         d->close();
     }
 
-    QString MacMIDIInput::currentConnection()
+    MIDIConnection MacMIDIInput::currentConnection()
     {
         return d->m_currentInput;
     }
@@ -347,5 +357,15 @@ namespace rt {
         return d->m_thruEnabled && d->m_out != 0;
     }
 
+    QStringList MacMIDIInput::getDiagnostics()
+    {
+        return d->m_diagnostics;
+    }
+
+    bool MacMIDIInput::getStatus()
+    {
+        return d->m_status;
+    }
+
 }} // namespace drumstick::rt
 
