@@ -1,10 +1,10 @@
 /*
     Drumstick RT Backend
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -20,7 +20,6 @@
 #include "maccommon.h"
 
 #include <QObject>
-#include <QDebug>
 #include <QString>
 #include <QStringList>
 #include <QByteArray>
@@ -42,10 +41,12 @@ namespace rt {
         MIDIEndpointRef m_destination;
         bool m_clientFilter;
 
-        QString m_currentOutput;
+        MIDIConnection m_currentOutput;
         QString m_publicName;
         QStringList m_excludedNames;
-        QStringList m_outputDevices;
+        QList<MIDIConnection> m_outputDevices;
+        bool m_status;
+        QStringList m_diagnostics;
 
         MacMIDIOutputPrivate():
             m_client(0),
@@ -58,25 +59,36 @@ namespace rt {
             internalCreate(DEFAULT_PUBLIC_NAME);
         }
 
+        void registerStatus(const QString& context, const OSStatus status)
+        {
+            if (status != noErr) {
+                m_diagnostics << QString("%1 error: %2").arg(context, status);
+                m_diagnostics << getErrorTextFromOSStatus(status);
+            }
+        }
+
         void internalCreate(CFStringRef name)
         {
             OSStatus result = noErr;
+            m_status = false;
+            m_diagnostics.clear();
             result = MIDIClientCreate( name, NULL, NULL, &m_client );
             if ( result != noErr ) {
-                qDebug() << "MIDIClientCreate() error:" << result;
+                registerStatus("MIDIClientCreate()", result);
                 return;
             }
             result = MIDISourceCreate( m_client, name, &m_endpoint );
             if ( result != noErr ) {
-                qDebug() << "MIDISourceCreate() error:" << result;
+                registerStatus("MIDISourceCreate()", result);
                 return;
             }
             result = MIDIOutputPortCreate( m_client, name, &m_port );
             if (result != noErr) {
-                qDebug() << "MIDIOutputPortCreate() error:" << result;
+                registerStatus("MIDIOutputPortCreate()", result);
                 return;
-            }
+            }            
             reloadDeviceList(true);
+            m_status = (result == noErr);
         }
 
         virtual ~MacMIDIOutputPrivate()
@@ -87,27 +99,30 @@ namespace rt {
         void internalDispose()
         {
             OSStatus result = noErr;
+            m_status = false;
+            m_diagnostics.clear();
             if (m_port != 0) {
                 result = MIDIPortDispose( m_port );
                 if (result != noErr) {
-                    qDebug() << "MIDIPortDispose() error:" << result;
+                    registerStatus("MIDIPortDispose()", result);
                     m_port = 0;
                 }
             }
             if (m_endpoint != 0) {
                 result = MIDIEndpointDispose( m_endpoint );
                 if (result != noErr) {
-                    qDebug() << "MIDIEndpointDispose() err:" << result;
+                    registerStatus("MIDIEndpointDispose()", result);
                     m_endpoint = 0;
                 }
             }
             if (m_client != 0) {
                 result = MIDIClientDispose( m_client );
                 if (result != noErr) {
-                    qDebug() << "MIDIClientDispose() error:" << result;
+                    registerStatus("MIDIClientDispose()", result);
                     m_client = 0;
                 }
             }
+            m_status = (result == noErr);
         }
 
         void setPublicName(QString name)
@@ -130,7 +145,7 @@ namespace rt {
                 if (dest != 0) {
                     QString name = getEndpointName(dest);
                     if ( m_clientFilter &&
-                         name.contains(QLatin1String("IAC"), Qt::CaseSensitive) )
+                         name.contains(QStringLiteral("IAC"), Qt::CaseSensitive) )
                         continue;
                     if ( name.contains (m_publicName) )
                         continue;
@@ -140,13 +155,14 @@ namespace rt {
                             break;
                         }
                     }
-                    if (!excluded)
-                        m_outputDevices << name;
+                    if (!excluded) {
+                        m_outputDevices << MIDIConnection(name, i);
+                    }
                 }
             }
-            if (!m_currentOutput.isEmpty() && m_destination != 0 &&
+            if (!m_currentOutput.first.isEmpty() && m_destination != 0 &&
                 !m_outputDevices.contains(m_currentOutput)) {
-                m_currentOutput.clear();
+                m_currentOutput = MIDIConnection();
                 m_destination = 0;
             }
         }
@@ -158,28 +174,17 @@ namespace rt {
                 MIDISend(m_port, m_destination, events);
         }
 
-        bool open(const QString &name)
+        bool open(const MIDIConnection& conn)
         {
-            int index;
-            QStringList allOutputDevices;
-            int num = MIDIGetNumberOfDestinations();
-            for (int i = 0; i < num; ++i) {
-                MIDIEndpointRef dest = MIDIGetDestination( i );
-                if (dest != 0)
-                   allOutputDevices << getEndpointName( dest );
-            }
-            index = allOutputDevices.indexOf(name);
-            if (index < 0)
-                return false;
-            m_destination = MIDIGetDestination( index );
-            m_currentOutput = name;
+            m_destination = MIDIGetDestination( conn.second.toInt() );
+            m_currentOutput = conn;
             return true;
         }
 
         void close()
         {
             m_destination = 0;
-            m_currentOutput.clear();
+            m_currentOutput = MIDIConnection();
         }
     };
 
@@ -200,7 +205,7 @@ namespace rt {
 
     QString MacMIDIOutput::backendName()
     {
-        return QLatin1Literal("CoreMIDI");
+        return QLatin1String("CoreMIDI");
     }
 
     QString MacMIDIOutput::publicName()
@@ -213,7 +218,7 @@ namespace rt {
         d->setPublicName(name);
     }
 
-    QStringList MacMIDIOutput::connections(bool advanced)
+    QList<MIDIConnection> MacMIDIOutput::connections(bool advanced)
     {
         d->reloadDeviceList(advanced);
         return d->m_outputDevices;
@@ -224,7 +229,7 @@ namespace rt {
         d->m_excludedNames = conns;
     }
 
-    void MacMIDIOutput::open(QString name)
+    void MacMIDIOutput::open(const MIDIConnection& name)
     {
         d->open(name);
     }
@@ -234,58 +239,13 @@ namespace rt {
         d->close();
     }
 
-    QString MacMIDIOutput::currentConnection()
+    MIDIConnection MacMIDIOutput::currentConnection()
     {
         return d->m_currentOutput;
     }
 
     /* Realtime MIDI slots */
 
-    /*void MacMIDIOutput::allNotesOff()
-    {
-        quint8 data[3];
-        quint8 buf[2048];
-        MIDIPacketList* pktlist = (MIDIPacketList*) &buf;
-        MIDIPacket* packet = MIDIPacketListInit(pktlist);
-        for(int chan = 0; chan < MIDI_CHANNELS && packet != NULL; ++chan) {
-            data[0] = MIDI_STATUS_CONTROLCHANGE | (chan & 0x0f);
-            data[1] = MIDI_CTL_ALL_NOTES_OFF;
-            data[2] = 0;
-            packet = MIDIPacketListAdd(pktlist, sizeof(buf), packet, 0,
-                      sizeof(data), data);
-            if (packet != NULL) {
-                data[1] = MIDI_CTL_ALL_SOUNDS_OFF;
-                packet = MIDIPacketListAdd(pktlist, sizeof(buf), packet, 0,
-                           sizeof(data), data);
-            }
-        }
-        if (packet != NULL)
-            d->sendEvents(pktlist);
-    }*/
-
-    /*void MacMIDIOutput::resetControllers()
-    {
-        quint8 data[3];
-        quint8 buf[2048];
-        MIDIPacketList* pktlist = (MIDIPacketList*) &buf;
-        MIDIPacket* packet = MIDIPacketListInit(pktlist);
-        for(int chan = 0; chan < MIDI_CHANNELS && packet != NULL; ++chan) {
-            data[0] = MIDI_STATUS_CONTROLCHANGE | (chan & 0x0f);
-            data[1] = MIDI_CTL_RESET_CONTROLLERS;
-            data[2] = 0;
-            packet = MIDIPacketListAdd(pktlist, sizeof(buf), packet, 0,
-                      sizeof(data), data);
-            if (packet != NULL) {
-                data[1] = MIDI_CTL_MSB_MAIN_VOLUME;
-                data[2] = 100;
-                packet = MIDIPacketListAdd(pktlist, sizeof(buf), packet, 0,
-                           sizeof(data), data);
-            }
-        }
-        if (packet != NULL)
-            d->sendEvents(pktlist);
-    }*/
-
     void MacMIDIOutput::sendNoteOn(int chan, int note, int vel)
     {
         quint8 data[3];
@@ -408,4 +368,14 @@ namespace rt {
             d->sendEvents(&pktlist);
     }
 
+    QStringList MacMIDIOutput::getDiagnostics()
+    {
+        return d->m_diagnostics;
+    }
+
+    bool MacMIDIOutput::getStatus()
+    {
+        return d->m_status;
+    }
+
 }}
