@@ -1,10 +1,10 @@
 /*
     Drumstick RT Mac OSX Backend
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,9 +16,9 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <QDebug>
 #include <QSettings>
 #include "macsynth.h"
+#include "maccommon.h"
 
 #include <CoreServices/CoreServices.h>
 #include <AudioUnit/AudioUnit.h>
@@ -34,18 +34,23 @@ namespace rt {
     private:
         AUGraph m_graph;
         AudioUnit m_synthUnit;
-        QString m_connection;
+        MIDIConnection m_connection;
         QString m_soundfont_dls;
         bool m_default_dls;
         bool m_reverb_dls;
+    public:
+        bool m_status;
+        QStringList m_diagnostics;
 
     public:
         explicit MacSynthOutputPrivate():
             m_graph(nullptr),
-            m_synthUnit(nullptr)
+            m_synthUnit(nullptr),
+            m_default_dls(true),
+            m_reverb_dls(false)
         {
             //qDebug() << Q_FUNC_INFO;
-            m_connection.clear();
+            m_connection = MIDIConnection();
         }
 
         ~MacSynthOutputPrivate()
@@ -54,6 +59,14 @@ namespace rt {
             stop();
         }
 
+        void registerStatus(const QString& context, const OSStatus status)
+        {
+            if (status != noErr) {
+                m_diagnostics << QString("%1 error: %2").arg(context).arg(status);
+                m_diagnostics << getErrorTextFromOSStatus(status);
+            }
+        }
+
         bool useDefaultDls() const
         {
             return m_default_dls;
@@ -90,7 +103,7 @@ namespace rt {
             }
         }
 
-        QString currentConnection()
+        MIDIConnection currentConnection()
         {
             return m_connection;
         }
@@ -103,6 +116,8 @@ namespace rt {
             AUNode synthNode = 0;
             AUNode outputNode = 0;
             AUNode limiterNode = 0;
+            m_status = false;
+            m_diagnostics.clear();
 
             //qDebug() << Q_FUNC_INFO;
             if (m_graph == nullptr) {
@@ -155,6 +170,9 @@ namespace rt {
                                                       0, &url, sizeof(url));
                         registerStatus( "AudioUnitSetProperty(SoundBankURL)", result);
                         CFRelease(url);
+                        if (result != noErr) {
+                            return;
+                        }
                     }
                 }
 
@@ -172,13 +190,13 @@ namespace rt {
                 }
 
                 for (uint i = 0; i < 16; ++i) {
-                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CTL_MSB_MAIN_VOLUME,100,0);
+                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CONTROL_MSB_MAIN_VOLUME,100,0);
                     registerStatus( "MusicDeviceMIDIEvent", result );
-                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CTL_REVERB_SEND,100,0);
+                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CONTROL_REVERB_SEND,100,0);
                     registerStatus( "MusicDeviceMIDIEvent", result );
-                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CTL_MSB_BANK_SELECT,0,0);
+                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CONTROL_MSB_BANK_SELECT,0,0);
                     registerStatus( "MusicDeviceMIDIEvent", result );
-                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CTL_LSB_BANK_SELECT,0,0);
+                    result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_CONTROLCHANGE+i, MIDI_CONTROL_LSB_BANK_SELECT,0,0);
                     registerStatus( "MusicDeviceMIDIEvent", result );
                     result = MusicDeviceMIDIEvent(m_synthUnit, MIDI_STATUS_PROGRAMCHANGE+i, 0,0,0);
                     registerStatus( "MusicDeviceMIDIEvent", result );
@@ -190,26 +208,33 @@ namespace rt {
                     return;
                 }
             }
-            m_connection = QString(QLatin1String(PRETTY_NAME));
+            m_status = (result == noErr);
+            m_connection = MIDIConnection(QStringLiteral(PRETTY_NAME), QStringLiteral(PRETTY_NAME));
         }
 
         void stop ()
         {
             OSStatus result;
+            m_status = false;
+            m_diagnostics.clear();
             //qDebug() << Q_FUNC_INFO;
             if (m_graph != nullptr) {
                 result = AUGraphStop(m_graph);
-                if (result != noErr)
-                    qWarning() << "AUGraphStop() err:" << result;
+                if (result != noErr) {
+                    registerStatus("AUGraphStop()", result);
+                }
                 result = AUGraphClose(m_graph);
-                if (result != noErr)
-                    qWarning() << "AUGraphClose() err:" << result;
+                if (result != noErr) {
+                    registerStatus("AUGraphClose()", result);
+                }
                 result = DisposeAUGraph(m_graph);
-                if (result != noErr)
-                    qWarning() << "DisposeAUGraph() err:" << result;
+                if (result != noErr) {
+                    registerStatus("DisposeAUGraph()", result);
+                }
                 m_graph = nullptr;
             }
-            m_connection.clear();
+            m_connection = MIDIConnection();
+            m_status = true;
         }
 
         void initialize ( QSettings *settings )
@@ -225,14 +250,6 @@ namespace rt {
             settings->endGroup();
         }
 
-        void registerStatus(const QString& context, const long status)
-        {
-            if (status != noErr) {
-                qWarning() << context << "err:" << status;
-            } //else {
-            //    qDebug() << context;
-            //}
-        }
 
         void sendStatusEvent(uint status, uint data1, uint data2)
         {
@@ -263,12 +280,12 @@ namespace rt {
 
     QString MacSynthOutput::backendName()
     {
-        return QLatin1String(PRETTY_NAME);
+        return QStringLiteral(PRETTY_NAME);
     }
 
     QString MacSynthOutput::publicName()
     {
-        return QLatin1String(PRETTY_NAME);
+        return QStringLiteral(PRETTY_NAME);
     }
 
     void MacSynthOutput::setPublicName(QString name)
@@ -276,10 +293,10 @@ namespace rt {
         Q_UNUSED(name)
     }
 
-    QStringList MacSynthOutput::connections(bool advanced)
+    QList<MIDIConnection> MacSynthOutput::connections(bool advanced)
     {
         Q_UNUSED(advanced)
-        return QStringList(QLatin1String(PRETTY_NAME));
+        return QList<MIDIConnection>{MIDIConnection(QStringLiteral(PRETTY_NAME),QStringLiteral(PRETTY_NAME))};
     }
 
     void MacSynthOutput::setExcludedConnections(QStringList conns)
@@ -287,7 +304,7 @@ namespace rt {
         Q_UNUSED(conns)
     }
 
-    void MacSynthOutput::open(QString name)
+    void MacSynthOutput::open(const MIDIConnection& name)
     {
         Q_UNUSED(name)
         //qDebug() << Q_FUNC_INFO;
@@ -300,7 +317,7 @@ namespace rt {
         d->stop();
     }
 
-    QString MacSynthOutput::currentConnection()
+    MIDIConnection MacSynthOutput::currentConnection()
     {
         return d->currentConnection();
     }
@@ -379,4 +396,14 @@ namespace rt {
         d->sendStatusEvent(static_cast<unsigned>(status), 0, 0);
     }
 
+    QStringList MacSynthOutput::getDiagnostics()
+    {
+        return d->m_diagnostics;
+    }
+
+    bool MacSynthOutput::getStatus()
+    {
+        return d->m_status;
+    }
+
 }}
