@@ -1,10 +1,10 @@
 /*
     Drumstick RT (realtime MIDI In/Out)
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,28 +16,24 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <QObject>
-#include <QFile>
 #include <QDir>
-#include <QDebug>
+#include <QFile>
+#include <QObject>
 
-#include "ossinput_p.h"
 #include "ossinput.h"
+#include "ossinput_p.h"
 
-namespace drumstick {
-namespace rt {
-
-static QString DEFAULT_PUBLIC_NAME(QLatin1String("MIDI In"));
+namespace drumstick { namespace rt {
 
 OSSInputPrivate::OSSInputPrivate(QObject *parent) : QObject(parent),
     m_inp(qobject_cast<OSSInput *>(parent)),
-    m_out(0),
-    m_device(0),
-    m_notifier(0),
-    m_parser(0),
+    m_out(nullptr),
+    m_device(nullptr),
+    m_notifier(nullptr),
+    m_parser(nullptr),
     m_thruEnabled(false),
     m_advanced(false),
-    m_publicName(DEFAULT_PUBLIC_NAME)
+    m_publicName(OSSInput::DEFAULT_PUBLIC_NAME)
 {
     reloadDeviceList();
 }
@@ -57,40 +53,40 @@ void OSSInputPrivate::reloadDeviceList(bool advanced)
     m_inputDevices.clear();
     QFileInfoList listInfo = dir.entryInfoList();
     foreach(const QFileInfo &info, listInfo) {
-        m_inputDevices << info.absoluteFilePath();
+        m_inputDevices << MIDIConnection(info.baseName(), info.absoluteFilePath());
     }
 }
 
-void OSSInputPrivate::open(QString portName)
+void OSSInputPrivate::open(const MIDIConnection& portName)
 {
-    QFile *f = new QFile(portName);
+    QFile *f = new QFile(portName.second.toString());
     m_currentInput = portName;
     m_device = f;
     m_device->open( QIODevice::ReadOnly | QIODevice::Unbuffered );
     m_notifier = new QSocketNotifier(f->handle(), QSocketNotifier::Read);
     m_parser = new MIDIParser(m_inp);
     m_buffer.clear();
-    connect(m_notifier, SIGNAL(activated(int)), this, SLOT(processIncomingMessages(int)));
+    connect(m_notifier, &QSocketNotifier::activated, this, &OSSInputPrivate::processIncomingMessages);
     //qDebug() << Q_FUNC_INFO << portName;
 }
 
 void OSSInputPrivate::close()
 {
-    if (m_device != 0) {
+    if (m_device != nullptr) {
         m_device->close();
         delete m_notifier;
         delete m_device;
         delete m_parser;
-        m_device = 0;
-        m_parser = 0;
+        m_device = nullptr;
+        m_parser = nullptr;
     }
-    m_currentInput.clear();
+    m_currentInput = MIDIConnection();
 }
 
 void OSSInputPrivate::setMIDIThruDevice(MIDIOutput* device)
 {
     m_out = device;
-    if (m_parser != 0) {
+    if (m_parser != nullptr) {
         m_parser->setMIDIThruDevice(device);
     }
 }
@@ -99,10 +95,11 @@ void OSSInputPrivate::processIncomingMessages(int)
 {
     char ch;
     m_device->getChar(&ch);
-    if (m_parser != 0) {
+    if (m_parser != nullptr) {
         uchar uch = static_cast<unsigned>(ch);
         m_parser->parse(uch);
     }
 }
 
-}}
+} // namespace rt
+} // namespace drumstick
