@@ -1,10 +1,10 @@
 /*
     Drumstick RT (realtime MIDI In/Out)
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -44,14 +44,14 @@ public:
     bool m_thruEnabled;
     bool m_advanced;
     QString m_publicName;
-    QString m_currentInput;
-    QStringList m_inputDevices;
+    MIDIConnection m_currentInput;
+    QList<MIDIConnection> m_inputDevices;
     QStringList m_excludedNames;
     QByteArray m_buffer;
 
-    OSSInputPrivate(QObject *parent = 0);
+    explicit OSSInputPrivate(QObject *parent = nullptr);
     void reloadDeviceList(bool advanced = false);
-    void open(QString portName);
+    void open(const MIDIConnection& portName);
     void close();
     //void parse();
     void setMIDIThruDevice(MIDIOutput* device);
