@@ -1,10 +1,10 @@
 /*
     Drumstick RT (realtime MIDI In/Out)
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,16 +16,14 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <QFile>
 #include <QDir>
-#include <QDebug>
+#include <QFile>
 
 #include "ossoutput.h"
 
-namespace drumstick {
-namespace rt {
+namespace drumstick { namespace rt {
 
-static QString DEFAULT_PUBLIC_NAME(QLatin1String("MIDI Out"));
+const QString OSSOutput::DEFAULT_PUBLIC_NAME = QStringLiteral("MIDI Out");
 
 class OSSOutput::OSSOutputPrivate
 {
@@ -33,13 +31,13 @@ public:
     bool m_advanced;
     QIODevice *m_device;
     QString m_publicName;
-    QString m_currentOutput;
-    QStringList m_outputDevices;
+    MIDIConnection m_currentOutput;
+    QList<MIDIConnection> m_outputDevices;
     QStringList m_excludedNames;
 
     OSSOutputPrivate() :
         m_advanced(false),
-        m_device(0),
+        m_device(nullptr),
         m_publicName(DEFAULT_PUBLIC_NAME)
     {
         reloadDeviceList();
@@ -65,26 +63,26 @@ public:
         m_outputDevices.clear();
         QFileInfoList listInfo = dir.entryInfoList();
         foreach(const QFileInfo &info, listInfo) {
-            m_outputDevices << info.absoluteFilePath();
+            m_outputDevices << MIDIConnection(info.baseName(), info.absoluteFilePath());
         }
     }
 
-    void open(QString portName)
+    void open(const MIDIConnection& portName)
     {
         //qDebug() << Q_FUNC_INFO << portName;
-        m_device = new QFile(portName);
+        m_device = new QFile(portName.second.toString());
         m_device->open(QIODevice::WriteOnly | QIODevice::Unbuffered);
         m_currentOutput = portName;
     }
 
     void close()
     {
-        if (m_device != 0) {
+        if (m_device != nullptr) {
             m_device->close();
             delete m_device;
-            m_device = 0;
+            m_device = nullptr;
         }
-        m_currentOutput.clear();
+        m_currentOutput = MIDIConnection();
     }
 
     void sendMessage(int m0)
@@ -116,8 +114,8 @@ public:
 
     void sendMessage(const QByteArray& message )
     {
-        if (m_device == 0) {
-            qDebug() << "qfile is null";
+        if (m_device == nullptr) {
+            //qDebug() << "device is null";
             return;
         }
         m_device->write(message);
@@ -141,7 +139,7 @@ void OSSOutput::initialize(QSettings *settings)
 
 QString OSSOutput::backendName()
 {
-    return QLatin1String("OSS");
+    return QStringLiteral("OSS");
 }
 
 QString OSSOutput::publicName()
@@ -154,7 +152,7 @@ void OSSOutput::setPublicName(QString name)
     d->m_publicName = name;
 }
 
-QStringList OSSOutput::connections(bool advanced)
+QList<MIDIConnection> OSSOutput::connections(bool advanced)
 {
     d->reloadDeviceList(advanced);
     return d->m_outputDevices;
@@ -165,7 +163,7 @@ void OSSOutput::setExcludedConnections(QStringList conns)
     Q_UNUSED(conns)
 }
 
-void OSSOutput::open(QString name)
+void OSSOutput::open(const MIDIConnection& name)
 {
     d->open(name);
 }
@@ -175,7 +173,7 @@ void OSSOutput::close()
     d->close();
 }
 
-QString OSSOutput::currentConnection()
+MIDIConnection OSSOutput::currentConnection()
 {
     return d->m_currentOutput;
 }
@@ -226,4 +224,5 @@ void OSSOutput::sendSystemMsg(const int status)
     d->sendMessage(status);
 }
 
-}}
+} // namespace rt
+} // namespace drumstick
