@@ -1,10 +1,10 @@
 /*
     Drumstick RT Windows Backend
-    Copyright (C) 2009-2019 Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2009-2021 Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,7 +16,6 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <QDebug>
 #include <QStringList>
 #include <QByteArray>
 #include <QVarLengthArray>
@@ -25,6 +24,10 @@
 #include <mmsystem.h>
 #include "winmidioutput.h"
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+#define hex Qt::hex
+#endif
+
 namespace drumstick {
 namespace rt {
 
@@ -34,7 +37,7 @@ namespace rt {
         quint8 data[sizeof(DWORD)];
     };
 
-    static QLatin1Literal DEFAULT_PUBLIC_NAME = QLatin1Literal("MIDI Out");
+    static QLatin1String DEFAULT_PUBLIC_NAME = QLatin1String("MIDI Out");
 
     void CALLBACK midiCallback( HMIDIOUT hmo,
                                 UINT wMsg,
@@ -47,11 +50,13 @@ namespace rt {
         HMIDIOUT m_handle;
         bool m_clientFilter;
         QString m_publicName;
-        QString m_currentOutput;
-        QMap<int,QString> m_outputDevices;
+        MIDIConnection m_currentOutput;
+        QList<MIDIConnection> m_outputDevices;
         MIDIHDR m_midiSysexHdr;
         QByteArray m_sysexBuffer;
         QStringList m_excludedNames;
+        bool m_status;
+        QStringList m_diagnostics;
 
         WinMIDIOutputPrivate():
             m_handle(nullptr),
@@ -71,17 +76,27 @@ namespace rt {
             m_clientFilter = !advanced;
 
             for ( dev = 0; dev < max; ++dev) {
+                bool excluded = false;
                 res = midiOutGetDevCaps( dev, &deviceCaps, sizeof(MIDIOUTCAPS));
                 if (res != MMSYSERR_NOERROR)
                     break;
-                if (m_clientFilter && (deviceCaps.wTechnology == MOD_MAPPER))
+                if (m_clientFilter && (deviceCaps.wTechnology == MOD_MAPPER || dev == MIDI_MAPPER))
                     continue;
     #if defined(UNICODE)
                 devName = QString::fromWCharArray(deviceCaps.szPname);
     #else
                 devName = QString::fromLocal8Bit(deviceCaps.szPname);
     #endif
-                m_outputDevices[dev] = devName;
+                foreach (const QString& n, m_excludedNames) {
+                    if (devName.startsWith(n)) {
+                        excluded = true;
+                        break;
+                    }
+                }
+
+                if (!excluded) {
+                    m_outputDevices << MIDIConnection(devName, dev);
+                }
             }
             if (!m_clientFilter) {
                 dev = MIDI_MAPPER;
@@ -92,7 +107,7 @@ namespace rt {
     #else
                     devName = QString::fromLocal8Bit(deviceCaps.szPname);
     #endif
-                    m_outputDevices[dev] = devName;
+                    m_outputDevices << MIDIConnection(devName, dev);
                 }
             }
         }
@@ -104,50 +119,39 @@ namespace rt {
             }
         }
 
-        int deviceIndex( const QString& newOutputDevice )
-        {
-            int index = -1;
-            QMap<int,QString>::ConstIterator it;
-            for( it = m_outputDevices.constBegin();
-                 it != m_outputDevices.constEnd(); ++it ) {
-                if (it.value() == newOutputDevice) {
-                    index = it.key();
-                    break;
-                }
-            }
-            return index;
-        }
-
-        void open(QString name)
+        void open(const MIDIConnection& conn)
         {
             MMRESULT res;
-            int dev = -1;
-
+            m_status = false;
+            m_diagnostics.clear();
             if (m_handle != nullptr)
                 close();
             reloadDeviceList(!m_clientFilter);
-            dev = deviceIndex(name);
-            if (dev > -1) {
-                res = midiOutOpen( &m_handle, dev, (DWORD_PTR) midiCallback, (DWORD_PTR) this, CALLBACK_FUNCTION);
-                if (res == MMSYSERR_NOERROR)
-                    m_currentOutput = name;
-                else
-                    qDebug() << "midiStreamOpen() err:" << mmErrorString(res);
+            if (!conn.first.isEmpty()) {
+                res = midiOutOpen( &m_handle, conn.second.toInt(), (DWORD_PTR) midiCallback, (DWORD_PTR) this, CALLBACK_FUNCTION);
+                if (res == MMSYSERR_NOERROR) {
+                    m_currentOutput = conn;
+                    m_status = true;
+                } else {
+                    logError("midiStreamOpen()", res);
+                }
             }
         }
 
         void close()
         {
             MMRESULT res;
+            m_status = false;
+            m_diagnostics.clear();
             if (m_handle != nullptr) {
                 res = midiOutReset( m_handle );
                 if (res != MMSYSERR_NOERROR)
-                    qDebug() << "midiOutReset() err:" << mmErrorString(res);
+                    logError("midiOutReset()", res);
                 res = midiOutClose( m_handle );
                 if (res == MMSYSERR_NOERROR)
-                    m_currentOutput.clear();
+                    m_currentOutput = MIDIConnection();
                 else
-                    qDebug() << "midiStreamClose() err:" << mmErrorString(res);
+                    logError("midiStreamClose()", res);
                 m_handle = nullptr;
             }
         }
@@ -157,7 +161,7 @@ namespace rt {
             MMRESULT res;
             res = midiOutUnprepareHeader( m_handle, lpMidiHdr, sizeof(MIDIHDR) );
             if (res != MMSYSERR_NOERROR)
-                qDebug() << "midiOutUnprepareHeader() err:" << mmErrorString(res);
+                logError("midiOutUnprepareHeader()", res);
             if ((lpMidiHdr->dwFlags & MHDR_ISSTRM) == 0)
                 return; // sysex header?
         }
@@ -167,7 +171,7 @@ namespace rt {
             MMRESULT res;
             res = midiOutShortMsg( m_handle, packet.dwPacket );
             if ( res != MMSYSERR_NOERROR )
-                qDebug() << "midiOutShortMsg() err:" << mmErrorString(res);
+                logError("midiOutShortMsg()", res);
         }
 
         void sendSysexEvent(const QByteArray& data)
@@ -181,11 +185,11 @@ namespace rt {
             m_midiSysexHdr.dwUser = 0;
             res = midiOutPrepareHeader( m_handle, &m_midiSysexHdr, sizeof(MIDIHDR) );
             if (res != MMSYSERR_NOERROR)
-                qDebug() << "midiOutPrepareHeader() err:" << mmErrorString(res);
+                logError("midiOutPrepareHeader()", res);
             else {
                 res = midiOutLongMsg( m_handle, &m_midiSysexHdr, sizeof(MIDIHDR) );
                 if (res != MMSYSERR_NOERROR)
-                    qDebug() << "midiOutLongMsg() err:" << mmErrorString(res);
+                    logError("midiOutLongMsg()", res);
             }
         }
 
@@ -195,7 +199,7 @@ namespace rt {
     #ifdef UNICODE
             WCHAR buffer[1024];
             midiOutGetErrorText(err, &buffer[0], sizeof(buffer));
-            errstr = QString::fromUtf16((const ushort*)buffer);
+            errstr = QString::fromWCharArray(buffer);
     #else
             char buffer[1024];
             midiOutGetErrorText(err, &buffer[0], sizeof(buffer));
@@ -203,6 +207,11 @@ namespace rt {
     #endif
             return errstr;
         }
+
+        void logError(const QString context, const MMRESULT code)
+        {
+            m_diagnostics << QString("%1 error: %2 (%3)").arg(context).arg(code).arg(mmErrorString(code));
+        }
     };
 
 
@@ -212,22 +221,21 @@ namespace rt {
                                 DWORD_PTR dwParam1,
                                 DWORD_PTR dwParam2)
     {
-        //Q_UNUSED(hmo)
+        Q_UNUSED(hmo)
         Q_UNUSED(dwParam2)
-
         WinMIDIOutput::WinMIDIOutputPrivate* obj = (WinMIDIOutput::WinMIDIOutputPrivate*) dwInstance;
         switch( wMsg ) {
         case MOM_DONE:
             obj->doneHeader( (LPMIDIHDR) dwParam1 );
             break;
         case MOM_OPEN:
-            qDebug() << "Open output" << hmo;
+            obj->m_diagnostics << "Open output";
             break;
         case MOM_CLOSE:
-            qDebug() << "Close output" << hmo;
+            obj->m_diagnostics << "Close output";
             break;
         default:
-            qDebug() << "unknown output message:" << hex << wMsg;
+            obj->m_diagnostics << QString("unknown output message: %1").arg(wMsg);
             break;
         }
     }
@@ -261,10 +269,10 @@ namespace rt {
         d->setPublicName(name);
     }
 
-    QStringList WinMIDIOutput::connections(bool advanced)
+    QList<MIDIConnection> WinMIDIOutput::connections(bool advanced)
     {
         d->reloadDeviceList(advanced);
-        return d->m_outputDevices.values();
+        return d->m_outputDevices;
     }
 
     void WinMIDIOutput::setExcludedConnections(QStringList conns)
@@ -272,7 +280,7 @@ namespace rt {
         d->m_excludedNames = conns;
     }
 
-    void WinMIDIOutput::open(QString name)
+    void WinMIDIOutput::open(const MIDIConnection& name)
     {
         d->open(name);
     }
@@ -282,7 +290,7 @@ namespace rt {
         d->close();
     }
 
-    QString WinMIDIOutput::currentConnection()
+    MIDIConnection WinMIDIOutput::currentConnection()
     {
         return d->m_currentOutput;
     }
@@ -362,4 +370,13 @@ namespace rt {
         d->sendSysexEvent(data);
     }
 
+    QStringList WinMIDIOutput::getDiagnostics()
+    {
+        return d->m_diagnostics;
+    }
+
+    bool WinMIDIOutput::getStatus()
+    {
+        return d->m_status;
+    }
 }}
