@@ -1,10 +1,10 @@
 /*
     MIDI Sequencer C++ library
-    Copyright (C) 2006-2019, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
@@ -23,7 +23,7 @@
 #include <QtGui>
 #endif
 
-const QString DEFVAL("f");
+const QString DrumGridModel::DEFVAL = QStringLiteral("f");
 
 DrumGridModel::DrumGridModel(QObject *parent)
     : QAbstractTableModel(parent),
@@ -38,11 +38,11 @@ DrumGridModel::DrumGridModel(QObject *parent)
 void DrumGridModel::loadKeyNames()
 {
     m_keyNames.clear();
-    m_keyNames[46] = QLatin1String("Open HH");
-    m_keyNames[42] = QLatin1String("Closed HH");
-    m_keyNames[39] = QLatin1String("Hand Claps");
-    m_keyNames[38] = QLatin1String("Snare Drum");
-    m_keyNames[36] = QLatin1String("Bass Drum");
+    m_keyNames[46] = QStringLiteral("Open HH");
+    m_keyNames[42] = QStringLiteral("Closed HH");
+    m_keyNames[39] = QStringLiteral("Hand Claps");
+    m_keyNames[38] = QStringLiteral("Snare Drum");
+    m_keyNames[36] = QStringLiteral("Bass Drum");
 }
 
 void DrumGridModel::fillSampleData()
