@@ -1,10 +1,10 @@
 /*
     MIDI Sequencer C++ library
-    Copyright (C) 2006-2019, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
@@ -37,37 +37,34 @@
 #include <drumstick/alsaqueue.h>
 #include <drumstick/subscription.h>
 
-using namespace drumstick;
-
 class QDumpMIDI : public QObject
 {
     Q_OBJECT
-
 public:
     QDumpMIDI();
     virtual ~QDumpMIDI();
 
-    void dumpEvent(SequencerEvent* ev);
+    void dumpEvent(drumstick::ALSA::SequencerEvent* ev);
     void subscribe(const QString& portName);
     void stop();
     bool stopped();
     void run();
 
 public slots:
-    void subscription(MidiPort* port, Subscription* subs);
+    void subscription( drumstick::ALSA::MidiPort* port, drumstick::ALSA::Subscription* subs );
 
 #ifdef USE_QEVENTS
 protected:
     virtual void customEvent( QEvent *ev );
 #else
-    void sequencerEvent( SequencerEvent* ev );
+    void sequencerEvent( drumstick::ALSA::SequencerEvent* ev );
 #endif
 
 private:
-    MidiClient* m_Client;
-    MidiPort* m_Port;
+    drumstick::ALSA::MidiClient* m_Client;
+    drumstick::ALSA::MidiPort* m_Port;
 #ifdef WANT_TIMESTAMPS
-    MidiQueue* m_Queue;
+    drumstick::ALSA::MidiQueue* m_Queue;
 #endif
     bool m_Stopped;
     QReadWriteLock m_mutex;
