@@ -1,12 +1,12 @@
 /*
     Standard MIDI File dump program
-    Copyright (C) 2006-2019, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     Based on midifile.c by Tim Thompson, M.Czeiszperger and Greg Lee
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
@@ -18,22 +18,27 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <cstdlib>
+#include "dumpsmf.h"
+#include <QCommandLineParser>
+#include <QCoreApplication>
+#include <QFileInfo>
 #include <QObject>
 #include <QString>
-#include <QCoreApplication>
-#include <QTextStream>
 #include <QTextCodec>
-#include <QFileInfo>
-#include <QCommandLineParser>
+#include <QTextStream>
+#include <cstdlib>
 #include <drumstick/qsmf.h>
-#include "cmdversion.h"
-#include "dumpsmf.h"
 
-const QString PGM_NAME("drumstick-dumpsmf");
-const QString PGM_DESCRIPTION("Drumstick command line utility for decoding SMF (Standard MIDI) files");
-static QTextStream cout(stdout, QIODevice::WriteOnly);
-static QTextStream cerr(stderr, QIODevice::WriteOnly);
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+#define right Qt::right
+#define left Qt::left
+#define endl Qt::endl
+#endif
+
+QTextStream cout(stdout, QIODevice::WriteOnly);
+QTextStream cerr(stderr, QIODevice::WriteOnly);
+
+using drumstick::File::QSmf;
 
 QSpySMF::QSpySMF():
     m_currentTrack(0), m_engine(nullptr), m_rc(0)
@@ -41,33 +46,36 @@ QSpySMF::QSpySMF():
     QTextCodec *codec = QTextCodec::codecForName("UTF-8");
     m_engine = new QSmf(this);
     m_engine->setTextCodec(codec);
-    connect(m_engine, SIGNAL(signalSMFHeader(int,int,int)), this, SLOT(headerEvent(int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFTrackStart()), this, SLOT(trackStartEvent()));
-    connect(m_engine, SIGNAL(signalSMFTrackEnd()), this, SLOT(trackEndEvent()));
-    connect(m_engine, SIGNAL(signalSMFNoteOn(int,int,int)), this, SLOT(noteOnEvent(int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFNoteOff(int,int,int)), this, SLOT(noteOffEvent(int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFKeyPress(int,int,int)), this, SLOT(keyPressEvent(int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFCtlChange(int,int,int)), this, SLOT(ctlChangeEvent(int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFPitchBend(int,int)), this, SLOT(pitchBendEvent(int,int)));
-    connect(m_engine, SIGNAL(signalSMFProgram(int,int)), this, SLOT(programEvent(int,int)));
-    connect(m_engine, SIGNAL(signalSMFChanPress(int,int)), this, SLOT(chanPressEvent(int,int)));
-    connect(m_engine, SIGNAL(signalSMFSysex(const QByteArray&)), this, SLOT(sysexEvent(const QByteArray&)));
-    connect(m_engine, SIGNAL(signalSMFSeqSpecific(const QByteArray&)), this, SLOT(seqSpecificEvent(const QByteArray&)));
-    connect(m_engine, SIGNAL(signalSMFMetaUnregistered(int, const QByteArray&)), this, SLOT(metaMiscEvent(int, const QByteArray&)));
-    //connect(m_engine, SIGNAL(signalSMFVariable(const QByteArray&)), this, SLOT(variableEvent(const QByteArray&)));
-    connect(m_engine, SIGNAL(signalSMFSequenceNum(int)), this, SLOT(seqNum(int)));
-    connect(m_engine, SIGNAL(signalSMFforcedChannel(int)), this, SLOT(forcedChannel(int)));
-    connect(m_engine, SIGNAL(signalSMFforcedPort(int)), this, SLOT(forcedPort(int)));
-    connect(m_engine, SIGNAL(signalSMFText(int,const QString&)), this, SLOT(textEvent(int,const QString&)));
-    connect(m_engine, SIGNAL(signalSMFendOfTrack()), this, SLOT(endOfTrackEvent()));
-    connect(m_engine, SIGNAL(signalSMFTimeSig(int,int,int,int)), this, SLOT(timeSigEvent(int,int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFSmpte(int,int,int,int,int)), this, SLOT(smpteEvent(int,int,int,int,int)));
-    connect(m_engine, SIGNAL(signalSMFKeySig(int,int)), this, SLOT(keySigEvent(int,int)));
-    connect(m_engine, SIGNAL(signalSMFTempo(int)), this, SLOT(tempoEvent(int)));
-    connect(m_engine, SIGNAL(signalSMFError(const QString&)), this, SLOT(errorHandler(const QString&)));
+    connect(m_engine, &QSmf::signalSMFHeader, this, &QSpySMF::headerEvent);
+    connect(m_engine, &QSmf::signalSMFTrackStart, this, &QSpySMF::trackStartEvent);
+    connect(m_engine, &QSmf::signalSMFTrackEnd, this, &QSpySMF::trackEndEvent);
+    connect(m_engine, &QSmf::signalSMFNoteOn, this, &QSpySMF::noteOnEvent);
+    connect(m_engine, &QSmf::signalSMFNoteOff, this, &QSpySMF::noteOffEvent);
+    connect(m_engine, &QSmf::signalSMFKeyPress, this, &QSpySMF::keyPressEvent);
+    connect(m_engine, &QSmf::signalSMFCtlChange, this, &QSpySMF::ctlChangeEvent);
+    connect(m_engine, &QSmf::signalSMFPitchBend, this, &QSpySMF::pitchBendEvent);
+    connect(m_engine, &QSmf::signalSMFProgram, this, &QSpySMF::programEvent);
+    connect(m_engine, &QSmf::signalSMFChanPress, this, &QSpySMF::chanPressEvent);
+    connect(m_engine, &QSmf::signalSMFSysex, this, &QSpySMF::sysexEvent);
+    connect(m_engine, &QSmf::signalSMFSeqSpecific, this, &QSpySMF::seqSpecificEvent);
+    connect(m_engine, &QSmf::signalSMFMetaUnregistered, this, &QSpySMF::metaMiscEvent);
+    connect(m_engine, &QSmf::signalSMFSequenceNum, this, &QSpySMF::seqNum);
+    connect(m_engine, &QSmf::signalSMFforcedChannel, this, &QSpySMF::forcedChannel);
+    connect(m_engine, &QSmf::signalSMFforcedPort, this, &QSpySMF::forcedPort);
+    connect(m_engine, &QSmf::signalSMFText, this, &QSpySMF::textEvent);
+    connect(m_engine, &QSmf::signalSMFendOfTrack, this, &QSpySMF::endOfTrackEvent);
+    connect(m_engine, &QSmf::signalSMFTimeSig, this, &QSpySMF::timeSigEvent);
+    connect(m_engine, &QSmf::signalSMFSmpte, this, &QSpySMF::smpteEvent);
+    connect(m_engine, &QSmf::signalSMFKeySig, this, &QSpySMF::keySigEvent);
+    connect(m_engine, &QSmf::signalSMFTempo, this, &QSpySMF::tempoEvent);
+    connect(m_engine, &QSmf::signalSMFError, this, &QSpySMF::errorHandler);
     cout.setRealNumberNotation(QTextStream::FixedNotation);
     cout.setRealNumberPrecision(4);
+#if (QT_VERSION < QT_VERSION_CHECK(6,0,0))
     cout.setCodec(codec);
+#else
+    cout.setEncoding(QStringConverter::Utf8);
+#endif
 }
 
 void QSpySMF::dump(const QString& chan, const QString& event,
@@ -246,11 +254,13 @@ int QSpySMF::numErrors()
 
 int main(int argc, char **argv)
 {
+    const QString PGM_NAME = QStringLiteral("drumstick-dumpsmf");
+    const QString PGM_DESCRIPTION = QStringLiteral("Drumstick command line utility for decoding SMF (Standard MIDI) files");
+
     QSpySMF spy;
-    QString file;
     QCoreApplication app(argc, argv);
     QCoreApplication::setApplicationName(PGM_NAME);
-    QCoreApplication::setApplicationVersion(PGM_VERSION);
+    QCoreApplication::setApplicationVersion(QStringLiteral(QT_STRINGIFY(VERSION)));
 
     QCommandLineParser parser;
     parser.setApplicationDescription(PGM_DESCRIPTION);
