@@ -1,10 +1,10 @@
 /*
     Virtual Piano test using the MIDI Sequencer C++ library
-    Copyright (C) 2006-2019, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -23,30 +23,23 @@
 #include <QCloseEvent>
 #include <drumstick/rtmidiinput.h>
 #include <drumstick/rtmidioutput.h>
-
 #include "ui_vpiano.h"
-#include "vpianoabout.h"
-#include "connections.h"
-#include "preferences.h"
-
-using namespace drumstick::rt;
 
 class VPiano : public QMainWindow
 {
     Q_OBJECT
 public:
-    VPiano( QWidget * parent = 0, Qt::WindowFlags flags = 0 );
+    VPiano( QWidget* parent = nullptr, Qt::WindowFlags flags = Qt::Window);
     virtual ~VPiano();
-    void closeEvent(QCloseEvent *event);
-    void findInput(QString name, QList<MIDIInput*> &inputs);
-    void findOutput(QString name, QList<MIDIOutput*> &outputs);
+    void showEvent(QShowEvent *event) override;
+    void closeEvent(QCloseEvent *event) override;
+    void setPortableConfig(const QString fileName = QString());
 
 public slots:
     void readSettings();
     void writeSettings();
 
     void slotAbout();
-    void slotAboutQt();
     void slotConnections();
     void slotPreferences();
 
@@ -55,23 +48,27 @@ public slots:
     void slotNoteOff(const int midiNote, const int vel);
     void slotNoteOff(const int chan, const int note, const int vel);
 
+    void slotChangeFont();
+    void slotNameOrientation(QAction* action);
+    void slotNameVisibility(QAction* action);
+    void slotNameVariant(QAction* action);
+    void slotCentralOctave(QAction* action);
+    void slotStandardNames();
+    void slotCustomNames(bool sharps);
+    void slotNoteName(const QString& name);
+
 private:
-    MIDIInput * m_midiIn;
-    MIDIOutput* m_midiOut;
-    QString m_lastInputBackend;
-    QString m_lastOutputBackend;
-    QString m_lastInputConnection;
-    QString m_lastOutputConnection;
-    bool m_midiThru;
-    bool m_advanced;
+    void initialize();
+    void useCustomNoteNames();
+
+    QList<drumstick::rt::MIDIInput*> m_inputs;
+    QList<drumstick::rt::MIDIOutput*> m_outputs;
+    drumstick::rt::MIDIInput * m_midiIn;
+    drumstick::rt::MIDIOutput* m_midiOut;
     Ui::VPiano ui;
-    About dlgAbout;
-    Connections dlgConnections;
-    Preferences dlgPreferences;
-    QString m_nativeInput;
-    QString m_nativeOutput;
-    QString m_defaultInput;
-    QString m_defaultOutput;
+
+//  QStringList m_names_s{"do", "do♯", "re", "re♯", "mi", "fa", "fa♯", "sol", "sol♯", "la", "la♯", "si"};
+//  QStringList m_names_f{"do", "re♭", "re", "mi♭", "mi", "fa", "sol♭", "sol", "la♭", "la", "si♭", "si"};
 };
 
 #endif // VPIANO_H
